@@ -115,6 +115,18 @@ my %print_result_of = (
         }
         return @o;
     },
+    'update_program' => sub {
+        my $inp = shift;
+        my $res = $inp->{result};
+        my @o = ();
+        if (exists $inp->{error}) {
+            push @o, sprintf("ERROR: (update): %s", $inp->{error}->{message} );
+        }
+        if ($res) {
+            push @o, "update OK.";
+        }
+        return @o;
+    },
     'list_programs' => sub {
         my $inp = shift;
         my $res = $inp->{result};
@@ -333,6 +345,21 @@ sub console_input {
                 $heap->{cmd_sent}->{$id} = $cmd;
                 last SWITCH;
             };
+            ($input =~ /^update\s+(\w+)\s+(\{.*\})$/) and do {
+                my $id = next_id(); 
+                my $pname = $1;
+                my $p = $2;
+                eval {
+                    $p = JSON::XS::decode_json( $p );
+                };
+                if ($@) {
+                    $heap->{cli_wheel}->put("ERROR: ".clean_eval_error($@));
+                    last SWITCH;
+                }
+                $cmd = { "method" => "update_program", "params" => [ $pname, $p ], "id" => $id };
+                $heap->{cmd_sent}->{$id} = $cmd;
+                last SWITCH;
+            };
             ($input =~ /^remove\s+(\w+)$/) and do {
                 my $id = next_id(); 
                 $cmd = { "method" => "remove_program", "params" => [ $1 ], "id" => $id };
